<?php
/**
 * Classification edit action plugin
 * allows editing gained score
 * 
 * @author Tomas Kadlec <tomas@tomaskadlec.net>
 */
 
if(!defined('DOKU_INC')) die();
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
if(!defined('DOKU_PLUGIN_CLASSIFICATION')) define('DOKU_PLUGIN_CLASSIFICATION',DOKU_INC.'lib/plugins/classification');
require_once(DOKU_PLUGIN.'action.php');

 
class action_plugin_classification_edit extends DokuWiki_Action_Plugin {
	
	/**
	 * constructor
	 */
	function action_plugin_classification_edit() {
		// load config
		$this->loadConfig();
		// start localization
		$this->setupLocale();
		// pattern used widely in this plugin
		$this->pattern = '/~~CLASSIFICATION\:(?:0|1)\:[0-9]{1,4}\:[^~]*?~~/';
	}

	/**
	 * Information about the author
	 * @return array with required info
	 */
	function getInfo(){
		return array(
			'author' => 'Tomas Kadlec',
			'email'  => 'tomas@tomaskadlec.net',
			'date'   => '2009-02-25',
			'name'   => 'Classification (action plugin component)',
			'desc'   => 'Score editor.',
			'url'    => 'http://webing.felk.cvut.cz/project/webiki',
 		);
  }
 
  /**
   * Register its handlers with the DokuWiki's event controller
   */
  function register(&$controller) {
		$controller->register_hook('TPL_ACT_RENDER', 'BEFORE', $this, 'form', array());
		$controller->register_hook('IO_WIKIPAGE_WRITE', 'BEFORE', $this, 'save', array());
  }
  
  /**
   * Update event handling
   * @param $param - for convinience with dokuwiki
   */
  function form(&$event, $param = null) {
  	global $ID;
    global $conf;
    global $INFO;
    // firstly, we must know study type or even if we are supposed to change the form
		if (preg_match('/^'.$this->conf['classification_storage_prez'].':[0-9]{1,4}$/', ':'.$ID) && $INFO['exists'])
			$this->study = 0;
		else if (preg_match('/^'.$this->conf['classification_storage_komb'].':[0-9]{1,4}$/', ':'.$ID) && $INFO['exists'])
			$this->study = 1;
		// if this is not classification, stop!
		else
			return;
    // event handling
  	switch ($event->data) {
  		// EDIT - show form
  		case 'edit':
  			$this->load();
  			$this->renderForm();
  			// we must prevent default action (dokuwiki edit form)
  			$event->preventDefault();
  			break;
  		default:
  	}
  }
  
	function save(&$event, $param = null) {
  	global $ID;
  	$this->initialize();
  	//echo '<pre>'.print_r($event, true).'</pre><br/>ID:'.$ID;
  	//echo '<pre>'.print_r($_REQUEST, true).'</pre><br/>ID:'.$ID;
  	if (!isset($_REQUEST['classification_score_update']))
  		return;
  	// FIXME load class (in a dagerous way, fix it (e.g. use session?))
  	$file = utf8_encodeFN(DOKU_INC.'data/pages/'.str_replace(':', '/', cleanID($ID)).'.txt');
  	$data = file_get_contents($file);
  	if (preg_match($this->pattern, $data, $matches) > 0) {
			foreach ($matches as &$match) {
				list($cmd, $study, $class, $ser) = explode(':', trim($match,'~ \t'), 4);
				$classification = new Classification($study, $class, $ser);
				break;
			}
		}
		//if (isset($_REQUEST('score')) && is_array($_REQUEST['score'])) {
			ClassificationStructure::load(DOKU_CONF.($study == 0 ? $this->conf['classification_struct_prez'] : $this->conf['classification_struct_komb']), ClassificationStructure::$structure);
  		$classification->update($_REQUEST['score']);
		//}
  	$replace = "~~CLASSIFICATION:$study:$class:{$classification->export()}~~";
		$event->data[0][1] = preg_replace($this->pattern, $replace, $event->data[0][1]);
		//echo '<pre>'.print_r($event,true).'</pre>';
		// we want to save our "fake" ;) data, do not prevent default action
  	//$event->preventDefault();
  }
  
  private function load() {
  	global $ID;
		$this->initialize();
  	// FIXME locking!!! lock file
  	// load file
  	$this->file = utf8_encodeFN(DOKU_INC.'data/pages/'.str_replace(':', '/', cleanID($ID)).'.txt');
  	$this->data = file_get_contents($this->file);
		// multiple matches may occur but only first one will be processed
		if (preg_match($this->pattern, $this->data, $matches) > 0) {
			foreach ($matches as &$match) {
				list($cmd, $this->study, $this->class, $ser) = explode(':', trim($match,'~ \t'), 4);
				$this->classification = new Classification($this->study, $this->class, $ser);
				break;		
			}
		}
  	// load structure
  	ClassificationStructure::load(DOKU_CONF.($this->study == 0 ? $this->conf['classification_struct_prez'] : $this->conf['classification_struct_komb']), ClassificationStructure::$structure);
  }
  
  private function renderForm() {
  	global $ID;
  	$out .=
  		'<div class="level2">'.
  		'<form method="POST" action="'.DOKU_BASE.DOKU_SCRIPT.'">'.
  		'<div class="no"><input type="hidden" name="id" value="'.$ID.'"/><input type="hidden" name="classification_score_update" value="aaa"/>'.
  		'<input type="hidden" id="wiki__text" name="wikitext" value="'.htmlspecialchars(addslashes($this->data)).'"/>'.
  		'</div>'.
  		'<table class="inline"><thead><tr>';
  	$s = &ClassificationStructure::$structure;
		foreach($s['structure'] as &$column)
			if ($column[2] >= 0 && $column[2] < 4)
				$out .= "<th>{$column[1]}</th>";
  	$out .=
  		'</tr></thead><tbody>';
  	$students = &$this->classification->output();
  	// tabindex variables
  	$count = count($students); $row = 0;
  	foreach($students as &$student) {
			$out .= '<tr>';
			// tabindex variables
			$row++; $position = 0;
			foreach($s['structure'] as &$column) {
				$column_name = 'name="score['.$student['__info'][0].']['.$column[0].']"';
			 	switch ($column[2]) {
			 		case 0:
			 			$out .= "<td>{$student[$column[0]]}</td>";
			 			break;
			 		case 1:
			 			$tabindex = 
			 			$out .= '<td><input tabindex="'.($row + $count * $position++).'" class="newScore" type="text" value="'.$student[$column[0]].'" '.$column_name.'/></td>';
			 			break;
			 		case 3:
			 			$out .= '<td><input tabindex="'.($row + $count * $position++).'" type="text" value="'.$student[$column[0]].'" '.$column_name.'/></td>';
			 			break;
			 		// boolean
			 		case 2:
			 			$out .= '<td><input tabindex="'.($row + $count * $position++).'" type="checkbox" '.($student[$column[0]] == 1 ? 'checked' : '').'" '.$column_name.'/></td>';
			 			break;
			 	}
			}	
			$out .= '</tr>';
		}
		echo $out; 
		formSecurityToken();
		echo
  		'</tbody></table>'.
  		'<div class="buttons"><input class="button" type="submit" name="do[save]" value="'.$this->lang['formSave'].'" />'.
  		'<input class="button" type="submit" name="do[cancel]" value="'.$this->lang['formCancel'].'" /></div>'.
  		'</form></div>';
  }
    
  private function update(Classification &$c) {
  	
  }
  
  private function initialize() {
  	global $ID;
  	global $conf;
  	require_once(DOKU_PLUGIN_CLASSIFICATION.'/inc/classification_structure.php');
  	require_once(DOKU_PLUGIN_CLASSIFICATION.'/inc/classification.php');
  	
  }
  
}
?>