<?php
/**
 * Admin plugin for handling imports to classification
 * 
 * @author Tomas Kadlec <tomas@tomaskadlec.net>
 */

if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
if(!defined('DOKU_PLUGIN_CLASSIFICATION')) define('DOKU_PLUGIN_CLASSIFICATION',DOKU_INC.'lib/plugins/classification/');
require_once(DOKU_PLUGIN.'admin.php');
 
/**
 * Plugin class
 */
class admin_plugin_classification extends DokuWiki_Admin_Plugin {
	
	/** list of available tab modules (set staticaly in constructor) */
	private $tabs;
	/** reference to currently used tab */
	private $currentTab;
	/** data to render*/
	public $data;
	/** shared space for import* methods */
	private $import;
	/** classification structure */
	private $structure = NULL;
 
	/**
	 * Returns basic information about the plug-in required by Dokuwiki
	 * @return array with basic information
	 */
	function getInfo(){
		return array(
			'author' => 'Tomáš Kadlec',
			'email'  => 'tomas@tomaskadlec.net',
			'date'   => '2009-02-26',
			'name'   => 'Classification - admin part - define structure, import students',
			'desc'   => 'Plugin responsible for defining table structure and handling imports (CSV, URL/HTML)',
			'url'    => 'http://webing.felk.cvut.cz/project/webiki/wiki/classification/download',
		);
	}
	
	/**
	 * Returns sort number - used for sorting dokuwiki admin menu
	 * @return sort number for DW admin menu
	 */
	function getMenuSort() {
		return 999;
	}
	
	/**
	 * Plugin is available only for superuser
	 * @return always true
	 */
	function forAdminOnly() {
		return true;
	}
	
	/**
	 * Contstructor
	 */
	function admin_plugin_classification() {
		// load config
		$this->loadConfig();
		// start localization
		$this->setupLocale();
		// available tabs
		$this->tabs = array(
			'settings' => array(
				'title'  => 'tabSettings',
				'pos'		 => 'left',
				//'file'   => 'classification_settings.php',
				'tab'		 => 'settings',
				'handle' => 'handleSettings',
				'html'   => 'renderSettings',
			),
			'structurePrez' => array(
				'title'  => 'tabStructurePrez',
				'pos'		 => 'left',
				'file'   => 'classification_structure.php',
				'tab'		 => 'structurePrez',
				'handle' => 'handleStructure',
				'html'   => 'renderStructure',
				'ser'		 => DOKU_CONF.'/'.$this->conf['classification_struct_prez'],
			),
			'structureKomb' => array(
				'title'  => 'tabStructureKomb',
				'pos'		 => 'left',
				'file'   => 'classification_structure.php',
				'tab'		 => 'structureKomb',
				'handle' => 'handleStructure',
				'html'   => 'renderStructure',
				'ser'		 => DOKU_CONF.'/'.$this->conf['classification_struct_komb'],
			),
			'import' => array(
				'title' => 'tabImport',
				'pos'		=> 'left',
				'file'   => array('classification_structure.php', 'classification.php'),
				'tab'		=> 'import',
				'handle' => 'handleImport',
				'html'   => 'renderImport',
				'ser_prez' => DOKU_CONF.'/'.$this->conf['classification_struct_prez'],
				'ser_komb' => DOKU_CONF.'/'.$this->conf['classification_struct_komb'],
				'store_prez' => $this->conf['classification_storage_prez'],
				'store_komb' => $this->conf['classification_storage_komb'],
			),
			'help' => array(
				'title' => 'tabHelp',
				'pos'		=> 'right',
				'tab'		=> 'help',
				'handle' => null,
				'html'   => 'renderHelp',
			),
		);
		// default tab/command
		$this->tabs['default'] = &$this->tabs['help'];
	}
 
	/**
	 * Handle users requests
	 * @return void
	 */
	function handle() {
		// required global variables
		global $ID;
		// select command
		if (!isset($_REQUEST['tab']))
			$tab = 'default';
		else if (isset($_REQUEST['tab']) && isset($this->tabs[$_REQUEST['tab']]))
			$tab = $_REQUEST['tab'];
		else
			$tab = null;
		// select propper tab description
		if (isset($tab)) 
			$this->currentTab = &$this->tabs[$tab];
		else
			 $this->currentTab = null;
		// request handling
		if (!isset($this->currentTab)) {
			$this->error[] = 'errUnknownAction';
			return;
		}
		// load required files
		if (isset($this->currentTab['file']) && is_array($this->currentTab['file']))
			foreach ($this->currentTab['file'] as $file)
				require_once(DOKU_PLUGIN_CLASSIFICATION."/inc/$file");
		// or file
		else if (isset($this->currentTab['file']))
			require_once(DOKU_PLUGIN_CLASSIFICATION.'/inc/'.$this->currentTab['file']);
		// call real handler
		if (is_callable(array($this,$this->currentTab['handle'])))
			call_user_func(array($this,$this->currentTab['handle']));
	}
	/**
	 * Structure changes handler 
	 */
	private function handleStructure() {		
		// load data structure
		// FIXME handle nonexisting file
		ClassificationStructure::load($this->currentTab['ser'], ClassificationStructure::$structure);
		// select propper action
		if (isset($_REQUEST['add'])) {
			// validation
			$valid = true;
			$valid &= ClassificationStructure::validateIdentifier($_REQUEST['id']) 
				|| msg($this->lang['errDuplicateId'], -1);
			$valid &= ClassificationStructure::validateType($_REQUEST['type']) 
				|| msg($this->lang['errInvalidType'], -1);
			// handling
			if ($valid)
				ClassificationStructure::addColumn($_REQUEST['newNr'], $_REQUEST['id'], $_REQUEST['caption'], $_REQUEST['type'], $_REQUEST['defaultValue'], true) &&
				ClassificationStructure::save($this->currentTab['ser'], ClassificationStructure::$structure, 1);
		}
		else if (isset($_REQUEST['save'])) {
			//validation
			$valid = true;
			$valid &= ClassificationStructure::validateIdentifier($_REQUEST['id'], $_REQUEST['curNr']) 
				|| msg($this->lang['errDuplicateId'], -1);
			$valid &= ClassificationStructure::validateType($_REQUEST['type']) 
				|| msg($this->lang['errInvalidType'], -1);
			$valid &= ClassificationStructure::validateCurNr($_REQUEST['curNr'])
				|| msg($this->lang['errNonExistingItem'], -1);
			// handling
			if ($valid)
				ClassificationStructure::saveColumn($_REQUEST['curNr'], $_REQUEST['newNr'], $_REQUEST['id'], $_REQUEST['caption'], $_REQUEST['type'], $_REQUEST['defaultValue']) && 
				ClassificationStructure::save($this->currentTab['ser'], ClassificationStructure::$structure, 1);
		}
		else if (isset($_REQUEST['delete'])) {
			$valid = true;
			$valid &= ClassificationStructure::validateCurNr($_REQUEST['curNr'])
				|| msg($this->lang['errNonExistingItem'], -1);
			if ($valid)
				ClassificationStructure::deleteColumn($_REQUEST['curNr']) &&
				ClassificationStructure::save($this->currentTab['ser'], ClassificationStructure::$structure, 1);
		}
		$this->data = &ClassificationStructure::$structure;
	}
	
	/**
	 * Handle import of CSV data
	 */
	private function handleImport() {
		// define pattern
		$this->import['pattern'] = '/~~CLASSIFICATION\:(?:0|1)\:[0-9]{1,4}\:[^~]*?~~/';
		// start processing
		if (isset($_REQUEST['importCSVDaily']) || isset($_REQUEST['importURLDaily'])) {
			// load structure
			ClassificationStructure::load($this->currentTab['ser_prez'], ClassificationStructure::$structure);
			// prepare to load classes
			$this->import['ns'] = cleanID($this->currentTab['store_prez']);
			$this->import['dir'] = utf8_encodeFN(DOKU_INC.'data/pages/'.str_replace(':', '/', cleanID($this->currentTab['store_prez'])));
			$this->import['study'] = 0;
		}
		else if (isset($_REQUEST['importCSVDistance']) || isset($_REQUEST['importURLDistance'])) {
			// load structure
			ClassificationStructure::load($this->currentTab['ser_komb'], ClassificationStructure::$structure);
			// prepare to load classes
			$this->import['ns'] = cleanID($this->currentTab['store_komb']);
			$this->import['dir'] = utf8_encodeFN(DOKU_INC.'/data/pages/'.str_replace(':', '/', cleanID($this->currentTab['store_komb'])));
			$this->import['study'] = 1;
		}
		// error
		else {
			//msg($this->lang['errImportFailed'], -1);
			return false;
		}
		// initialize classes
		$classes = array();
		// load existing classes
		if (!isset($_REQUEST['importForce']))
			$this->importLoad($classes);
		// get new data
		if (isset($_REQUEST['importCSVDaily']) || isset($_REQUEST['importCSVDistance']))
			$students = $this->importCSV($_FILES['importCSVfile']['tmp_name']);
		if (isset($_REQUEST['importURLDaily']) || isset($_REQUEST['importURLDistance']))
			$students = $this->importURL($this->import['study']);
		// check && create needed dirs
		if (!file_exists($this->import['dir']))
			mkdir($this->import['dir'], 0700, true);
		// import
		if ($students !== false)
			$this->import($classes, $students);
		// save
		foreach ($classes as $class => &$data) {
			// modified? -> save
			if ($data['mod'] == 1) {
				$data['class']->ksort();
				$replace = "~~CLASSIFICATION:{$this->import['study']}:$class:{$data['class']->export()}~~";
				file_put_contents($data['path'], preg_replace($this->import['pattern'], $replace, $data['data']));
			}
			// don't forget to unlock pages
			unlock($data['id']);
		}
		//$this->data['echo'] = &$classes;
		msg($this->lang['importSuccessful'],1);
	}
	
/**
	 * Load already defined students
	 * @param &$classes - array of defined classes
	 * @return true on success, false otherwise
	 */
	private function importLoad(&$classes) {
		// cycle through classification directory
		if (is_dir($this->import['dir']) && ($dh = opendir($this->import['dir'])) !== false) {
			while (($file = readdir($dh)) !== false) {
				if (!preg_match('/^[0-9]{1,4}\.txt$/',$file))
					continue;
				else {
					$id = preg_replace('/\.txt/','', $file);
					if (checklock(":$ns:$id") === false) {
						// set lock
						lock(":{$this->import['ns']}:$id");
						// save necessary information
						$classes[$id] = array(
							'id' => "{$this->import['ns']}:$id",
							'path' => "{$this->import['dir']}/$file",
							'data' => @file_get_contents("{$this->import['dir']}/$file"),
						);
						// multiple matches may occur but only first one will be processed
						if (preg_match($this->import['pattern'], $classes[$id]['data'], $matches) > 0) {
							foreach ($matches as &$match) {
								list($cmd, $study, $class, $ser) = explode(':', trim($match,'~ \t'), 4);
								$classes[$id]['class'] = new Classification($study, $class, $ser);
								break;		
							}
						} // end match processing 
					} // end checklock
				} // end file to process
			} // end directory processing
			closedir($dh);
			return true;
    }
    return false;
	}
	
	/**
	 * parse CSV file
	 * @param $csvfile - path to CSV file
	 * @param $delimiter - field delimiter, default ':'
	 * @return array( par_c =>, name =>, login =>, email => ) or false when array is empty
	 */
	private function &importCSV($csvfile, $delimiter = ':') {
		// read file
		$csv = file_get_contents($csvfile);
		// FIXME csv import error handling - file read error 
		if ($csv === false)
			return array();
		$r = array();
		foreach (explode("\n", $csv) as $line) {
			list($login, $email, $name, $class) = explode(':', $line);
			if ($login != null)
				$r[] = array(
					'par_c' => html_entity_decode($class),
					'name' => html_entity_decode($name),
					'login' => html_entity_decode($login),
					'email' => html_entity_decode($email),
				);
		}
		if (!is_array($r) || count($r) <= 0)
			return false;
		return $r;
	}

	/**
	 * parse URL (web schedule)
	 * @param $lectures - import lectures
	 * @return array( par_c =>, name =>, login =>, email =>, par_p => ) or false when array is empty 
	 * @author Vojta Jirkovsky <jirkov1@fel.cvut.cz>
	 */
	private function &importURL($lectures = true) {
		require_once(DOKU_PLUGIN_CLASSIFICATION.'/inc/reader.php');
		require_once(DOKU_PLUGIN_CLASSIFICATION.'/inc/curl.php');
		// false - switch off lecture numbers
		return $students = get_all_students($_REQUEST['importURL'],$_REQUEST['importLogin'],$_REQUEST['importPassword'], $lectures);	
	}
	
	/**
	 * Generic student import method, expexted data format:
	 * array of arrays ( par_c =>, name =>, login =>, email => )
	 * @param $classes - array of defined classes
	 * @param $students - students to import
	 */
	private function import(&$classes, &$students) {
		foreach ($students as &$student) {
			// initialization
			$class = (isset($student['par_c']) ? $student['par_c'] : $student['par_p']);
			$login = $student['login'];
			$name = $student['name']; $email = $student['email'];
			// test if particular class already exists
			if (isset($classes[$class]['class']) && is_a($classes[$class]['class'], 'Classification')) {
				$c = &$classes[$class]['class'];
				//echo 'old class<pre>'.print_r($c,true).'</pre>';
			}
			else {
				try {
					$classes[$class]['id'] = "{$this->import['ns']}:$class";
					$classes[$class]['path'] = "{$this->import['dir']}/$class.txt"; // FIXME locking of newly created files
					$classes[$class]['data'] = "====== $class ======\n\n ~~CLASSIFICATION:{$this->import['study']}:$class:a~~\n\n";
					$classes[$class]['class'] = new Classification($this->import['study'], $class);
					$c = &$classes[$class]['class'];
				}
				catch (ClassificationClassOutOfRangeException $e) {
					msg($this->lang['exImportClassFailed'].': '.$class.' '.$this->lang['exImportStudentFailed'].': '.$login, 2);
					unset($classes[$class]);
					continue;
				}
			}
			// check, if student exists
			foreach($classes as &$aclass) {
				if ($aclass['class']->isStudent($login))
					continue 2; // skip this line
			}
			// try to import him
			try {
				$c->import($login, $email, $name, $class);
				// create student's namespace
				if (!file_exists(DOKU_INC.'/data/pages/student/'.$login)) {
					mkdir(DOKU_INC.'/data/pages/student/'.$login, 0700, true);
					// initial page
					$start = 
						"====== $login ======\n".
						"  * [[semestralka-zadani]]\n".
						"  * [[semestralka-dokumentace]]\n".
						"  * [[referat-zadani]]\n".
						"  * [[referat]]\n";
					file_put_contents(DOKU_INC.'/data/pages/student/'.$login.'/start.txt', $start);
					// necessary ACL
					$acl =
						"student:$login:*	$login 16\n".
						"student:$login:start	$login 1\n".
						"student:$login:semestralka-zadani	$login 4\n".
						"student:$login:referat-zadani	$login 4\n";
					file_put_contents(DOKU_CONF.'/acl.auth.php', $acl, FILE_APPEND);
				}
			}
			catch (ClassificationClassOutOfRangeException $e) {
				continue;
			}
			$classes[$class]['mod'] = 1;
		}
	}

	/**
	 * output appropriate HTML
	 * @return void
	 */
	function html() {
		// required global variables
		global $ID;
    // create instance od ClassificationAdminView
    require_once(DOKU_PLUGIN_CLASSIFICATION.'/inc/admin_view.php');
    $adminView = new ClassificationAdminView($this->currentTab['tab'], $this->lang, $ID, $this);
    // render openning div and tabs
    $adminView->renderTabs($this->tabs, $this->currentTab['tab']);
    // render right tab according to command
    $adminView->renderContent($this->currentTab['html'], $this->data);
		// closing div
		
		echo '</div>';
	}
 
}