<?php
/**
 * Class responsible for Classification admin view creation
 * 
 * @author Tomas Kadlec <tomas@tomaskadlec.net>
 */
class ClassificationAdminView {
	
	/** dokuwiki localization array */
	public $lang;
	/** base url */
	private $baseUrl;
	/** admin_plugin_classification */
	private $admin;
	
	/**
	 * Initialization of ClassificationAdminView
	 * @param $tab - current tab name
	 * @param $lang - dokuwiki $lang variable for localization
	 * @param $ID - base URL for anchor generation 
	 * @param $admin - instance of admin plugin (provides useful methods)
	 */
	public function __construct($tab, $lang, $ID, $admin) {
		$this->tab = $tab;
		$this->lang = $lang;
		$this->baseUrl = $ID;
		$this->admin = $admin;
	}
	
	/**
	 * Render tabs using defined CSS
	 * @param - tabs array (look at the structure!)
	 */
	public function renderTabs(array $tabs, $selected) {
		$out = '<div class="tabber"><div class="tabberheading">';
		foreach ($tabs as $key => $tab) {
			if ($key == 'default')
				continue;
			$style  = ($selected == $tab['tab'] ? ' selected' : '');
			$style .= ($tab['pos'] == 'right' ? ' right' : ' left' );
			$out .= "<div class=\"$style\"><a href=\"?do=admin&amp;page=classification&amp;tab={$tab['tab']}\">{$this->lang[$tab['title']]}</a></div>";
		}
		echo $out.'</div><div class="clearer">&nbsp;</div>';
	}
	
	/**
	 * Render data using specified method
	 * @param $method - callback resource to handle the operation (within this class)
	 * @param $data - data to render, can be null
	 * @return void
	 */
	public function renderContent($method, &$data = null) {
		echo '<div class="content">';
		if (is_callable(array($this,$method)))
			call_user_func(array($this,$method), $data);
		else
			msg($this->lang['errRenderHandlerFailed'], -1);
		echo '</div>';
	}
	
	public function renderStructure(array $data) {
		$types = &$data['types'];
		$structure = &$data['structure'];
		// current structure
		$out .=
			"<h2>{$this->lang['headStructure']}</h2>".
			'<div class="level2">'. 
			'<table class="inline"><thead><tr>'.
			"<th>{$this->lang['strNr']}</th>".
			"<th>{$this->lang['strId']}</th>".
			"<th>{$this->lang['strCaption']}</th>".
			"<th>{$this->lang['strType']}</th>". 
			"<th>{$this->lang['strDefaultValue']}</th>".     
			"<th colspan=\"2\">{$this->lang['strAction']}</th>".
			'</tr></thead><tbody>';
			// list of columns

			foreach ($structure as $id => $column) {
				$out .= 
					"<form action=\"&amp;do=admin&amp;page=classification&amp;tab={$this->tab}&amp;cmd=edit\" method=\"post\">".
					(formSecurityToken()).
					'<tr>'.
					'<td>'.
					"<input class=\"number\" type=\"text\" name=\"newNr\" value=\"$id\" />".
					"<input type=\"hidden\" name=\"curNr\" value=\"$id\" />".
					'</td>'.
					"<td><input type=\"text\" name=\"id\" value=\"{$column[0]}\" /></td>".
					"<td><input type=\"text\" name=\"caption\" value=\"{$column[1]}\" /></td>".
					'<td><select name="type">';
					foreach ($types as $typeID => $typeName) {
						$out .= 
							'<option value="'.$typeID.'" '.($column[2] == $typeID ? 'selected' : '' ).'>'.$typeName.'</option>';
					}
				$out .= 
					'</select></td>'.
					($column[2] != 4 ?
						"<td><input type=\"text\" name=\"defaultValue\" value=\"{$column[3]}\" /></td>" :
						"<td><textarea rows=\"1\" name=\"defaultValue\">{$column[3]}</textarea></td>" ).
					"<td><input type=\"submit\" name=\"save\" value=\"{$this->lang['formSave']}\" /></td>".
					"<td><input type=\"submit\" name=\"delete\" value=\"{$this->lang['formDelete']}\" /></td>".
					'</tr>'.
					'</form>';
			}
			// close table, form and structure definition div 
			$out .= '</table></div>';
			// new column
			$out .=
				"<h2>{$this->lang['headStructureNewColumn']}</h2>".
				'<div class="level2">'. 
				'<table class="inline"><thead><tr>'.
				"<th>{$this->lang['strNr']}</th>".
				"<th>{$this->lang['strId']}</th>".
				"<th>{$this->lang['strCaption']}</th>".
				"<th>{$this->lang['strType']}</th>". 
				"<th>{$this->lang['strDefaultValue']}</th>".     
				"<th>{$this->lang['strAction']}</th>".
				'</tr></thead><tbody>'.
				"<form action=\"&amp;do=admin&amp;page=classification&amp;tab={$this->tab}&amp;cmd=new\" method=\"post\">".
				(formSecurityToken()).
				'<tr>'.
				'<td>'.
				"<input class=\"number\" type=\"text\" name=\"newNr\" value=\"\" />".
				'</td>'.
				"<td><input type=\"text\" name=\"id\" value=\"\" /></td>".
				"<td><input type=\"text\" name=\"caption\" value=\"\" /></td>".
				'<td><select name="type">';
			foreach ($types as $typeID => $typeName) {
				$out .= 
					'<option value="'.$typeID.'">'.$typeName.'</option>';
			}
			$out .= 
				'</select></td>'.
				"<td><textarea rows=\"1\" name=\"defaultValue\"></textarea></td>".
				"<td><input type=\"submit\" name=\"add\" value=\"{$this->lang['formSave']}\" /></td>".
				'</tr>'.
				'</form>';
			// close table, form and structure definition div 
			$out .= '</table></div>';
			echo $out;
	}
	
	/**
	 * Render import forms
	 * @param $data - method does not care about the parametr, it is only for convinience with renderContent method
	 */
	private function renderImport(&$data = null) {
		$out =
			"<h2>{$this->lang['importCSVDaily']}</h2>". 
			'<div class="level2"><form action="" method="post" enctype="multipart/form-data">'.
			"<label>{$this->lang['formFile']}</label>".'<input class="import" type="file" name="importCSVfile" />'.
			"<label>{$this->lang['formForce']}</label>".'<input class="import" type="checkbox" name="importForce" />'.
			"<input type=\"submit\" name=\"importCSVDaily\" value=\"{$this->lang['formUpload']}\" />".
			'</form></div><div class="level2"><br /><hr /><br /></div>'.
			'<div class="level2"><form action="" method="post" enctype="multipart/form-data">'.
			"<label>{$this->lang['formLogin']}</label>".'<input class="import" type="text" name="importLogin" />'.
			"<label>{$this->lang['formPassword']}</label>".'<input class="import" type="password" name="importPassword" />'.
			"<label>{$this->lang['formURL']}</label>".'<input class="import" type="text" name="importURL" />'.
			"<label>{$this->lang['formForce']}</label>".'<input class="import" type="checkbox" name="importForce" />'.
			"<input type=\"submit\" name=\"importURLDaily\" value=\"{$this->lang['formUpload']}\" />".
			'</form></div>';
		$out .=
			"<h2>{$this->lang['importCSVDistance']}</h2>". 
			'<div class="level2"><form action="" method="post" enctype="multipart/form-data">'.
			"<label>{$this->lang['formFile']}</label>".'<input class="import" type="file" name="importCSVfile" />'.
			"<label>{$this->lang['formForce']}</label>".'<input class="import" type="checkbox" name="importForce" />'.
			"<input type=\"submit\" name=\"importCSVDistance\" value=\"{$this->lang['formUpload']}\" />".
			'</form></div><div class="level2"><br /><hr /><br /></div>'.
			'<div class="level2"><form action="" method="post" enctype="multipart/form-data">'.
			"<label>{$this->lang['formLogin']}</label>".'<input class="import" type="text" name="importLogin" />'.
			"<label>{$this->lang['formPassword']}</label>".'<input class="import" type="password" name="importPassword" />'.
			"<label>{$this->lang['formURL']}</label>".'<input class="import" type="text" name="importURL" />'.
			"<label>{$this->lang['formForce']}</label>".'<input class="import" type="checkbox" name="importForce" />'.
			"<input type=\"submit\" name=\"importURLDistance\" value=\"{$this->lang['formUpload']}\" />".
			'</form></div>';
		
		echo $out;
		//echo '<pre>'.print_r($data['echo'], true).'</pre>';
	}
	
	/**
	 * Render help in appropriate language or in english
	 * @param $data - method does not care about it
	 */
	private function renderHelp($data = null) {
		$this->admin->locale_xhtml('help');
	}
	
}
?>