<?php
/**
 * Class for handling ClassificationStructure
 * (model, singleton)
 * 
 * @require: PHP 5!
 * 
 * @author Tomas Kadlec <tomas@tomaskadlec.net>
 */

class ClassificationStructure { 
	
	/** ClassificationStructure structure **/
	public static $structure;
	/** singleton object */
	private static $singleton = NULL;
	/** data storage */
	private $classification;
	/** expression storage */
	private $expressions;
	
	/**
	 * delete column
	 * @param $curId - current column id
	 * @return true
	 */
	public static function deleteColumn($curNr) {
		unset(self::$structure['structure'][$curNr]);
		return true;
	}
	
/**
	 * add column - add column
	 * @param $nr - sort number
	 * @param $id - identifier - must be unique
	 * @param $caption - column caption
	 * @param $type - type (must be defined)
	 * @param $defaultValue
	 * @return true on success, false otherwise
	 */
	public static function addColumn($nr, $id, $caption, $type, $defaultValue) {
		$s = &self::$structure['structure'];
		// shift if necessary 
		if (is_numeric($nr) && isset($s[$nr]))
			self::shiftArray($s, $nr);
		// set position
		else if (!is_numeric($nr)) {
			end($s);
			$nr = key($s) + 1;
		}
		// if type is expression, prepare it
		if ($type == 4) $expr = self::compile($defaultValue); else $expr = null;
		// insert item
		$s[$nr] = array($id, $caption, $type, $defaultValue, $expr);
		return true;
	}
	
	/**
	 * save column - add/modify column
	 * @param $curNr - current sort number
	 * @param $newNr - requested sort number
	 * @param $id - identifier - must be unique
	 * @param $caption - column caption
	 * @param $type - type (must be defined)
	 * @param $defaultValue
	 * @param $shift - do shift the array (true), false - default
	 * @param $move - default false
	 * @return always true
	 */
	public static function saveColumn($curNr, $newNr, $id, $caption, $type, $defaultValue) {
		$s = &self::$structure['structure'];
		// if newNr is jibrash, set it as curNr
		if (!is_numeric($newNr))
			$newNr = $curNr; 
		// at first delete old item
		self::deleteColumn($curNr);
		// shift array
		if (is_numeric($newNr) && isset($s[$newNr]))
			self::shiftArray($s, $newNr);
		// if type is expression, prepare it
		if ($type == 4) $expr = self::compile($defaultValue); else $expr = null;
		// insert item
		$s[$newNr] = array($id, $caption, $type, $defaultValue, $expr);
		return true;
	}
	
	/**
	 * Compiles an expression
	 * @param $e - expression to compile
	 * FIXME - remove ", ' and similar dangerous chars from input
	 * FIXME - check input for only numerical, boolean and temporary cols 
	 */
	private static function compile($e) {
		$c = '';
		$tokens = preg_split('/([a-zA-Z][a-zA-Z0-9_]*)/', $e, -1, PREG_SPLIT_DELIM_CAPTURE);
		foreach ($tokens as &$token) {
			switch($token) {
				case 'if' :
				case 'else' :
				case 'for' :
					$c .= "$token";
					break;
				case 'Y' :
				case 'Z' :
				case 'N' :
					$c .= "'$token'";
					break;
				case 'A':
					$c .= "'A (výborně)'";
					break;
				case 'B':
					$c .= "'B (velmi dobře)'";
					break;
				case 'C':
					$c .= "'C (dobře)'";
					break;
				case 'D':
					$c .= "'D (uspokojivě)'";
					break;
				case 'E':
					$c .= "'E (dostatečně)'";
					break;
				case 'F':
					$c .= "'F (nedostatečně)'";
					break;
				default:
					$c .= preg_replace('([a-zA-Z][a-zA-Z0-9_]*)', '$this->students["--LOGIN--"]["\\0"]' , $token);
					break;
			}
		}
		unset ($tokens);
		return $c;
	}
		
	/**
	 * Shift array to empty requested position
	 * @param $array - array to shift
	 * @param $position - position
	 */
	private static function shiftArray(&$array, $position) {
		end($array);
		do {
			$array[key($array)+1] = current($array);
		} while (prev($array) != FALSE && key($array) >= $position);
	}
	
	/**
	 * Check if item at specified index exists
	 * @param $curNr - allowed one match at curNr position
	 * @return true if is false otherwise
	 */
	public static function validateCurNr($curNr) {
		if (isset(self::$structure['structure'][$curNr]))
				return true;
		return false;
	}
	
	/**
	 * Check if identifier is valid (it is not used)
	 * @param $id - identifier
	 * @param $curNr - allowed one match at curNr position
	 * @return true if is false otherwise
	 */
	public static function validateIdentifier($id, $curNr = null) {
		$s = &self::$structure['structure'];
		foreach ($s as $nr => $column)
			if (strcmp("{$column[0]}","$id") === 0 && ($curNr === null || intval($curNr) !== $nr)) {
				echo "nr: $nr curNr: $curNr b: ".($curNr !== $nr ? 1 : 0);
				return false;
			}
		return true;
	}
	
	/**
	 * Check if type is defined
	 * @param type - type
	 * @return true if is false otherwise
	 */
	public static function validateType($type) {
		$s = &self::$structure['types'];
		if (!is_numeric($type) && !isset($s[$type]))
			return false;
		return true;
	}
		
	/**
	 * Serialize
	 * @param $path - where to save
	 * @param $variable - variable to save
	 * @return false on failure, true otherwise
	 */
	public static function save($path, &$variable, $ksort = false) {
		if ($sort !== false)
			self::ksort($variable, $ksort);
		if ((file_exists($path) && !is_writable($path)) || !is_writable(dirname($path))) {
			msg("Failed saving data.", -1);
			return false;
		}
		if (!$handle = fopen($path, 'w'))
			return false;
		ksort($variable, SORT_NUMERIC);
		if (fwrite($handle, serialize($variable)) === FALSE || fclose($handle) === FALSE) {
			msg("Failed saving data.", -1);
			return false;
		}
		else {
			msg("Data successfully saved.", 1);
			return true;
		}
	}
	
	/**
	 * recursively ksort array
	 * @param $array - array to sort
	 * @param $ksort - depth
	 */
	private static function ksort(&$array, $ksort) {
		ksort($array);
		foreach($array as &$subarray)
			if (is_array($subarray) && $ksort > 0)
				self::ksort($subarray, $ksort - 1);
	}
	
	/**
	 * Unserialize
	 * @param $path - from where to load
	 * @param $variable - from where to load
	 * @return false on failure, true otherwise
	 */
	public static function load($path, &$variable) {
		if (!is_readable($path)) {
        return false;
    }
    $variable = unserialize(
    	file_get_contents($path)
    );
    // FIXME format cheks!
    return true;
	}
		

	private function __construct() {
		// path initialization
		if (!defined('DOKU_PLUGIN_PROJEKTY')) define('DOKU_PLUGIN_PROJEKTY',DOKU_INC.'lib/plugins/projekty/');
		// unserialization
	}
		
}

?>