<?php
/**
 * Syntax for classification
 * - mainly does rendering
 * 
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Tomáš Kadlec <tomas@tomaskadlec.net>
 */
// must be run within Dokuwiki
if(!defined('DOKU_INC')) die();
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
if(!defined('DOKU_PLUGIN_CLASSIFICATION')) define('DOKU_PLUGIN_CLASSIFICATION',DOKU_INC.'lib/plugins/classification/');
require_once(DOKU_PLUGIN.'syntax.php');

/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_classification extends DokuWiki_Syntax_Plugin {
  
	/**
	 * Constructior
	 */
	function syntax_plugin_classification(){
		$this->setupLocale();
		$this->loadConfig();
		$this->pattern = '~~CLASSIFICATION\:(?:0|1)\:[0-9]{1,4}\:[^~]*?~~';
	}

	/**
	 * Returns basic information about the plug-in required by Dokuwiki
	 * @return array with basic information
	 */
	function getInfo(){
		return array(
			'author' => 'Tomáš Kadlec',
			'email'  => 'tomas@tomaskadlec.net',
			'date'   => $this->version,
			'name'   => 'Classification - syntax part - tag <classification prez/komb cviceni= data= /> ',
			'desc'   => 'Plug-in provides tag for displaying classification',
			'url'    => 'http://webing.felk.cvut.cz/project/webiki/wiki/classification/start',
		);
	}

	/**
	 * Plugin mode type is substitution
	 * @return always string substitution
	 */
	function getType(){
		return 'substition';
	}
	
	/**
	 * Any plugins are allowed to process markup inside classification
	 * @return always null
	 *
  function getAllowedTypes() { 
		return array('disabled'); 
  }*/
	
	/**
	 * Syntax substitues for a table - it is a block mode.
	 * @return always string block
	 */
	function getPType(){
		return 'block';
	}

	/**
	 * Sort number
	 * @return sort number
	 */
	function getSort() {
		return 999;
	}
	
  /**
   * Set entry pattern
   * @param $mode - name of mode (dokuwiki internal)
   */
	function connectTo($mode) {
		$this->Lexer->addSpecialPattern($this->pattern,$mode,'plugin_classification');
	}

	/**
	 * Set exit pattern
	 *
	function postConnect() { 
		$this->Lexer->addExitPattern('</classification>','plugin_classification'); 
	}*/
	
	/**
	 * Handle the match
	 */
	function handle($match, $state, $pos, &$handler){
		// lexical analysis :)
		list($cmd, $study, $class, $ser) = explode(':', trim($match,'~ \t'), 4);
		// load ClassificationStructure
		require_once(DOKU_PLUGIN_CLASSIFICATION.'/inc/classification_structure.php');
		// load appropriate classification structure
		switch ($study) {
			case 0:
				ClassificationStructure::load(DOKU_CONF.'/'.$this->conf['classification_struct_prez'], $data['structure']);
				break;
			case 1:
				ClassificationStructure::load(DOKU_CONF.'/'.$this->conf['classification_struct_komb'], $data['structure']);
				break;
		}
		// load Classification class
		require_once(DOKU_PLUGIN_CLASSIFICATION.'/inc/classification.php');
		// instantiate
		try {
			// if classification data should be filtered using login, whole class must be available to the renderer and ~~NOCACHE~~ (xhtml) must be used.
			$data['classification'] = new Classification($study, $class, $ser);
		}
		catch (ClassificationCommonException $e) {
			msg ($this->lang[$e->getMessage], -1);
		}
		return $data;
	}

	/**
	 * Create output
	 */
	function render($mode, &$renderer, $data) {
		// required global variables
		global $INFO;
		// needed for unserialization of Classification class
		require_once(DOKU_PLUGIN_CLASSIFICATION.'/inc/classification.php');
		// test rendering mode
		if($mode == 'xhtml') {
			// start rendering
			$renderer->doc .= '<div class="level1"><table class="inline"	><thead><tr>';
			// render table heading and prepare array of column types
			foreach($data['structure']['structure'] as &$column)
				$renderer->doc .= "<th>{$column[1]}</th>";
			$renderer->doc .= "</tr></thead><tbody>";
			// filter data
			try {
				$c = ($INFO['writable'] ? $data['classification']->output() : $data['classification']->output($INFO['client']));
				foreach($c as &$student) {
					$renderer->doc .= '<tr>';
					foreach($data['structure']['structure'] as &$column) {
					 	switch ($column[2]) {
					 		// boolean
					 		case 2:
					 			$renderer->doc .= '<td>'.($student[$column[0]] == 1 ? 'A' : '-').'</td>';
					 			break;
					 		// student and link
					 		case 5:
					 			// FIXME calling parser is very inefficient this way (is there another possibility?)
					 			$renderer->doc .= '<td>'.(p_render($mode, p_get_instructions($student[$column[0]]),$info)).'</td>';
					 			break;
					 		// string (3)
					 		default:
								$renderer->doc .= "<td>{$student[$column[0]]}</td>";
					 	}
					}	
					$renderer->doc .= '</tr>';
				}
			}
			catch (ClassificationLoginNotFoundException $e) {
				msg($this->lang[$e->getMessage()], -1);
			}
			catch (ClassificationCommonException $e) {
				msg($this->lang[$e->getMessage()], -1);
			}
			$renderer->doc .= '</tbody></table></div>';
			//$renderer->doc .= '<pre>'.print_r($data, true).print_r($INFO,true).'</pre>';
    }
	}
}