<?php
/**
 * Plugin nspages : Displays nicely a list of the pages of a namespace
 * v1.6.1 - 2009-06-1
 *
 * Usage :
 * <nspages> presents the pages of the current namespace
 * <nspages namespace:subnamespace> presents the pages of the selected namespace
 * <nspages .subnamespace1:subnamespace2> presents the pages of the selected subnamespace
 * Moreover, some options may be used :
 * <nspages ... -subns> lists subnamespaces (not their contents)
 * <nspages ... -nopages> do not list the pages of the namespace (but this option is a bit useless if -subns isn't used either...)
 * <nspages ... -exclude:page1 -exclude:page2 -exclude:ns:> avoid listing pages "page1" and "page2", and own't list either the subnamespace "ns"
 * <nspages ... -exclude:[@page1 @page2 @ns:]> do the same than the above line (but is easier to write!)
 * <nspages ... -exclude> avoid listing the current page
 * <nspages ... -h1> or <nspages ... -title> prints the first h1 title of the pages, instead of their name. If there aren't such titles, the name of the page is used instead
 *
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Guillaume Turri <guillaume.turri@gmail.com>
 *
 * modified under the terms of the GPL by Daniel Schranz <xla@gmx.at>
 * date of change: February 4th 2009
 * changes made only to function render(...), but added the file style.css
 *    implemented a three-column output, similar to the way Wikipedia has styled it's category pages
 *
 * modified under the terms of the GPL by Ignacio Bergmann
 * date of change: 2009-06-01
 * introduction of Doku_Renderer_xhtml in the _printElement function
 */

if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');
require_once(DOKU_INC . 'inc/pageutils.php');// to use the getID() function
require_once(DOKU_INC . 'inc/auth.php');//to use the auth_aclcheck() function
require_once(DOKU_INC . 'inc/parser/xhtml.php');//to use the class Doku_Renderer_xhtml to render internal wiki links


/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_nspages extends DokuWiki_Syntax_Plugin {

    function getInfo() {
        return array(
            'author' => 'Guillaume Turri',
            'email' => 'guillaume.turri@gmail.com',
            'date' => '2009-04-22',
            'name' => 'nspages Plugin',
            'desc' => 'Presents a list of the pages in the selected namespace, using <nspages> or <nspages path_to_a_namespace>. Some options like -subns; -nopages; -exclude or -h1 may also be used. See http://www.dokuwiki.org/plugin:nspages to get more details',
            'url' =>  'http://www.dokuwiki.org/plugin:nspages');
    } // getInfo()

    function connectTo($aMode) {
        $this->Lexer->addSpecialPattern('<nspages ?[a-zA-Z0-9_.: -\[\]@]*>', $aMode, 'plugin_nspages');
    } // connectTo()

    function getSort() {
        return 999;
    } // getSort()

    function getType() {
        return 'substition';
    } // getType()

    function handle($match, $state, $pos, &$handler) {
        //Looking for the -subns  option
        $subns = (bool) preg_match("/-subns/i", $match);
        //Looking for the -nopages option
        $nopages = (bool) preg_match("/-nopages/i", $match);

        //Looking for excluded pages
          //--Checking if specified pages have to be excluded
        $exclude = array();
        preg_match_all("/-exclude:([-.a-zA-Z0-9_]*)/", $match, $allExclude);
        foreach($allExclude[1] as $page)
            array_push($exclude, $page.'.txt');
          //--Looking if the current page has to be excluded
        $id = getID();
        if(preg_match("/-exclude[^:]/", $match)){
            preg_match("/([-.a-zA-Z0-9_]*)$/", $id, $page);
            array_push($exclude, $page[1].'.txt');
        }

        //Looking for excluded subnamespaces
        $exclude_ns = array();
        preg_match_all("/-exclude:([-.a-zA-Z0-9_]*):/", $match, $allExclude);
        foreach($allExclude[1] as $ns)
            array_push($exclude_ns, $ns);

        //--Looking if the syntax -exclude[page1 page2] has been used
        if(preg_match("/-exclude:\[([- .a-zA-Z0-9_:@]*)\]/", $match, $ex)){
            $ex = str_replace('@', '', $ex[1]);
            $ex = explode(' ', $ex);
            foreach($ex as $temp){
                if($temp[strlen($temp)-1] == ':')
                    array_push($exclude_ns, substr($temp, 0, -1));
                else
                    array_push($exclude, $temp.'.txt');
            }
        }

        //Looking for the -h1 (or -title) option
        $title = (bool) preg_match("/-title/i",$match);
        $title = $title || (bool) preg_match("/-h1/i",$match);

        //Looking for the wanted namespace
        if(preg_match("/ [a-zA-Z0-9_.:]+/", $match, $wanted_ns) == 0)
            $wanted_ns='';
        else
            $wanted_ns = substr($wanted_ns[0], 1);//getting rid of the whitespace
        if($wanted_ns == '') $wanted_ns = '.';//ie : we want the current ns

        if($wanted_ns[0] == '.'){
            //computing the current ns (note : cf. getNS() from /pageutils.php
            $ns = $id;
            if(strpos($ns, ':') !== FALSE)
                for ( $i = strlen($ns) ; $i>0 ; $i-- ) {
                    if ( ':' == $ns[$i] ) {
                        $ns = substr($ns, 0, $i);
                        break;
                    }
                }
            else
                $ns = '';
        }
        else{
            $ns = '';   //case of an absolute path
        }

        //Computing the absolute path to the wanted namespace
        while( $wanted_ns != '' ){
            if( $wanted_ns[0] == '.' && $wanted_ns[1] == '.'){
                $wanted_ns = substr($wanted_ns, 2);
                if( $wanted_ns[0] == ':' ) $wanted_ns = substr($wanted_ns, 1);
                $ns = ereg_replace(":?[a-zA-Z0-9_]*$",'',$ns);
            }
            else if( $wanted_ns[0] == '.' || $wanted_ns[0] == ':'){
                $wanted_ns = substr($wanted_ns, 1);
                if( $wanted_ns[0] == ':' ) $wanted_ns = substr($wanted_ns, 1);
            }
            else{
                if( $wanted_ns[0] == ':' ) $wanted_ns = substr($wanted_ns, 1);
                ereg("^[a-zA-Z0-9_]*", $wanted_ns, $nom);
                $wanted_ns = ereg_replace("^[a-zA-Z0-9_]*", '', $wanted_ns);
                $ns .=':'.$nom[0];
            }
        }


        if( $ns[strlen($ns) - 1] != ':') $ns .= ':';
        if( $ns[0] == ':' ) $ns = substr($ns, 1);
        global $conf;
        $dir = $conf['savedir'].'/pages/'.str_replace(':','/',$ns); //The user may change the dokuwiki's default option, so it may not be './data'

        return array($dir, $ns, $subns, $nopages, $exclude, $exclude_ns, $title);
    } // handle()

    function render($mode, &$renderer, $data) {
        global $USERINFO;
        if($mode == 'xhtml'){
            $dossier = $data[0];
            $namespace = $data[1];
            $subns = $data[2];
            $nopages = $data[3];
            $exclude = $data[4];
            $exclude_ns = $data[5];
            $title = $data[6];
            $pages = array();
            $subnamespaces = array();

            //Getting the files
            if($dh = @opendir($dossier)){
                while ( ($file = readdir($dh)) != false ) {
                    if( $file != '.' && $file != '..' ){
                        if( is_file($dossier.$file) && !in_array($file, $exclude) && auth_aclcheck($namespace.substr($file, 0, -4), $_SERVER['REMOTE_USER'], $USERINFO['grps']) > 0){ //if the page isn't excluded, and if the user may read it. substr is to get rid of .txt
                            array_push($pages, array('name' => $title ? $this->_getTitle($dossier, $file) : substr($file, 0, -4), 'link' => substr($file, 0, -4))); //array(what will be display, what will be used as a link);
                        }
                        if( is_dir($dossier.$file) && !in_array($file, $exclude_ns) && auth_aclcheck($namespace.$file.':', $_SERVER['REMOTE_USER'], $USERINFO['grps']) > 0){
                            array_push($subnamespaces, array('name' => $file));
                        }
                    }
                }

                closedir($dh);

                //writting the output
                //--listing the subnamespaces (if needed)
                if( $subns ){
                    $this->_printNicely($renderer, $subnamespaces, $namespace, 'subns');
                }

                //--listing the pages
                if( !$nopages ){
                    $this->_printNicely($renderer, $pages, $namespace, 'page');
                }
            }
            else{
                $renderer->doc .= '<div>'.$this->getLang('doesntexist').$namespace.'</div>';
            }
                        //this is needed to make sure everything after the plugin is written below the output
                        $renderer->doc .= '<br class="catpageeofidx">';
            return TRUE;
        } else {
            return FALSE;
        }
    } // render()

    function _printNicely(&$renderer, $tab, $namespace, $type='page'){
        sort($tab);

        //calculate how many elements should be in the first and second column
        $collength = ceil( sizeof($tab) / 3 );
        $dblcollength = ceil( sizeof($tab) / 3 ) * 2;
        //if there are less than three elements keep them in one column (just for beauty-issues)
        if ( sizeof($tab) < 3 ){
            $collength = 10;
            $dblcollength = 10;
        }

        //use actpage to count how many pages we have already processed
        $actpage=0;

        //write this (localized) text as headline for the section
        $renderer->doc .= '<p class="catpageheadline">'.$this->getLang( ($type=='page') ? 'pagesinthiscat' : 'subcats' )."</p>";
        if( sizeof($tab) == 0){
            $renderer->doc .= '<p>'.$this->getLang( ($type=='page') ? 'nopages' : 'nosubns').'</p>';
        }
        else{
            $renderer->doc .= "\n".'<div class="catpagecol"><ul>';
            //firstchar stores the first character of the last added page
            $firstchar = strtoupper(substr($tab[0]['name'],0,1));
            //write the first index-letter
            $renderer->doc .= '<div class="catpagechars">'.$firstchar."</div>\n";

            foreach( $tab as $element ){
                //change to the next column if necessary
                if ($actpage == $collength || $actpage == $dblcollength) {
                    $renderer->doc .= "</ul></div>\n".'<div class="catpagecol"><ul>'."\n";
                    if ( strtoupper(substr($element['name'],0,1)) != $firstchar ) {
                        $firstchar = strtoupper(substr($element['name'],0,1));
                        $renderer->doc .= '<div class="catpagechars">'.$firstchar."</div>\n";
                    }
                    else {
                        $renderer->doc .= '<div class="catpagechars">'.$firstchar.$this->getLang('continued')."</div>\n";
                    }
                }
                //write the index-letters
                if (strtoupper(substr($element['name'],0,1))!=$firstchar) {
                    $firstchar = strtoupper(substr($element['name'],0,1));
                    $renderer->doc .= '<div class="catpagechars">'.$firstchar."</div>\n";
                }

                $this->_printElement($renderer, $element, $namespace, $type);
                $actpage++;
            }
            $renderer->doc .= "</ul></div>\n";
        }
    } // _printNicely()

    function _printElement(&$renderer, $element, $namespace, $type='page'){
        $xhmtlrender = new Doku_Renderer_xhtml();
        if( $type == 'page' ){
            $renderer->doc .= '<li class="level1"><div class="li">'.$xhmtlrender->internallink($namespace.$element['link'], $element['name'], NULL,true).'</div></li>';
        }
        else{  //Case of a subnamespace
            $renderer->doc .= '<li class="closed"><div class="li">'.$xhmtlrender->internallink($namespace.$element['name'].':', $element['name'], NULL,true).'</div></li>';
        }
    } // _printElement()

    /* Take the name of a page and look for the first h1 title
     * Returns the name of this title, or the name of the page if such a title doesn't exist in this page
     */
    function _getTitle($dossier, $page){
        $trouve = (bool) preg_match("/={6}[\t  ]*([^=]*)={6}/", file_get_contents($dossier.$page), $titre);
        $result = $trouve ? $titre[1] : substr($page, 0, -4); //getting rid of '.txt'
        $result[0] = strtoupper($result[0]); //To make sure sort() will work fine
        return $result;
    }

  //@}
} // class syntax_plugin_nspages
