<?php
/**
 * Permissioninfo: Displays group and user information and the permissions of users and groups
 * 
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Gabriel Birke <birke@d-scribe.de>
 */
 
if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'admin.php');
 
/**
 * All DokuWiki plugins to extend the admin function
 * need to inherit from this class
 */
class admin_plugin_permissioninfo extends DokuWiki_Admin_Plugin {

    /**
     * If information about which user is in which group is displayed
     */
    var $show_group_info = true;
 
    /**
     * return some info
     */
    function getInfo(){
      return array(
        'author' => 'Gabriel Birke',
        'email'  => 'birke@d-scribe.de',
        'date'   => '2007-10-10',
        'name'   => 'admin plugin permissioninfo',
        'desc'   => 'Displays group and user information and the permissions of users and groups',
        'url'    => 'http://wiki.splitbrain.org/plugin:permissioninfo',
      );
    }
 
    /**
     * return sort order for position in admin menu
     */
    function getMenuSort() {
      return 145;
    }
    
    /**
     * handle user request
     */
    function handle() {
        global $conf;
        $auth_class = "auth_".$conf['authtype'];
        // We don't check for existence of the auth class or success after instantiating - this was handled by auth.php
        $this->auth = new $auth_class();
        
        // If the auth class can't list users or groups, retrieve user and group information from ACL
        if($this->auth->canDo('getUsers'))
        {
            $getUserFunc  = array($this->auth, 'retrieveUsers');
        }
        else
        {
            // Can't determine user/group association from ACL
            $this->show_group_info = false;
            $getUserFunc = array($this, "_getUsersFromACL");
        }
        if($this->auth->canDo('getGroups'))
            $getGroupFunc = array($this->auth, 'retrieveGroups');
        else
            $getGroupFunc = array($this, '_getGroupsFromACL');
        // Collect user and group names
        $this->users = call_user_func($getUserFunc);
        $this->groups = call_user_func($getGroupFunc);

        // Get permissions for each group and set the data in $this->aclGroupPermissions
        $this->_aclGroupPermissions();

        // Get explicit user permissions from ACL and set the data in $this->aclUserPermissions
        $this->_aclUserPermissions();

        // Associate groups with users and set the data in $this->group2user
        $this->_group2user();

        // If we show permissions for an individual user, collect its permissions
        if(!empty($_REQUEST['show']) && !empty($_REQUEST['user']))
        {
            $this->_userPermissions($_REQUEST['user']);
        }
    }
 
    /**
     * output Overview page with groups or permissionpage for individual user, all
     * depending on $_REQUEST['show']
     */
    function html() {
        switch($_REQUEST['show'])
        {
            case 'userpermissions':
                $this->_showUserPermissions();
                break;
            case 'overview':
            default:
                $this->_groupOverview();
        }
    }

    /**
     * Shows an overview for users in groups and permissions assigned to groups
     */
    function _groupOverview()
    {
        $id = cleanID($this->getLang('menu'));
		$onload_js = "";
        ptln('<h1><a name="'.$id.'" id="'.$id.'">'.$this->getLang('menu')."</a></h1>");
        echo $this->locale_xhtml('help');
		
        foreach($this->groups as $gname => $g)
        {
            $original_name = $gname;
            $gname = urldecode($gname);
            // container for group information
            ptln('<div class="piContainer">');
            // print group header
            ptln('   <div class="piHeader">');
            ptln("      <h2>$gname</h2>");
            ptln("<span class='piTogglers'>", 6);
            // Toggle User View
            ptln('<a href="javascript:piToggleView(\'Users\', \''.$gname.
                '\')" id="piViewTogglerUsers'.$gname.'">'.$this->getLang('pi_hide_users').'</a>', 9);
            ptln("| ",9);
            // Toggle ACL View
            ptln('<a href="javascript:piToggleView(\'Permissions\', \''.$gname.
                '\')" id="piViewTogglerPermissions'.$gname.'">'.$this->getLang('pi_hide_permissions').'</a>', 9);
            ptln('</span>', 6);
            ptln('   </div>');
			$onload_js .= "piToggleView('Users', '$gname');\npiToggleView('Permissions', '$gname');\n";
				
            // print users in group
            ptln('   <div class="piUsers" id="piUsers'.$gname.'">');
            foreach($this->group2user[$original_name] as $u)
            {
                $url = wl($ID, array(
                    'do' => "admin",
                    'page' => $this->getPluginName(),
                    'show' => 'userpermissions',
                    'user' => $u
                ));
                $u_enc = auth_nameencode($u);
                $lnk = '<a href="'.$url.'"  title="'.$u.'" '.(!empty($this->aclUserPermissions[$u_enc])?'class="special"':"").'>';
                ptln('      <div>'.$lnk.$this->users[$u]['name'].'</a></div>');
            }
            ptln('<br style="clear:both;" />');
            ptln('   </div>');
            
            // print acl settings for this group 
            $this->_permissionTable($this->aclGroupPermissions[$original_name], "piPermissions".$gname);

            //end container
            ptln('</div>');
        }
		 
		ptln('<script type="text/javascript"><!--//--><![CDATA[//><!--');
		// JS vars for i18n
        ptln('piLang={}; // Init translation hash');
        ptln('piLang["show_users"] = "'.$this->getLang('pi_show_users').'";');
        ptln('piLang["hide_users"] = "'.$this->getLang('pi_hide_users').'";');
        ptln('piLang["show_permissions"] = "'.$this->getLang('pi_show_permissions').'";');
        ptln('piLang["hide_permissions"] = "'.$this->getLang('pi_hide_permissions').'";');
		// Close the containers with Javascript for an initial view that's more clear.
		ptln("addInitEvent(function(){\n".$onload_js."});");
		ptln('//--><!]]></script>');
    }

    /**
     * Show permissions for individual user, highlight permissions that were 
     * assigned explicitly to this user.
     */
    function _showUserPermissions()
    {
        $head = sprintf($this->getLang('pi_permissionfor'), $this->username);
        $id = cleanID($head);
        ptln('<h1><a name="'.$id.'" id="'.$id.'">'.$head."</a></h1>");
        echo $this->locale_xhtml('help_userpermissions');
        
        // Link to Overview
        $url =wl($ID, array(
            'do' => "admin",
            'page' => $this->getPluginName(),
            'show' => 'overview'
        ));
        ptln('<p class="piToOverview"><a href="'.$url.'">'.$this->getLang('pi_to_overview')."</a></p>");

        $this->_permissionTable($this->userPermissions, 'Userpermissions');
        //dbg($this->explicitUserPermissions);
    }

    /**
     * Print permissions for a user or group
     * @param array $acldata namespace/page_name=>permission pairs
     * @param string $id ID for the div that surrounds teh table
     */
    function _permissionTable($acldata, $id)
    {
        $displayed_permissions = array(
            AUTH_READ,
            AUTH_EDIT,
            AUTH_CREATE,
            AUTH_UPLOAD,
            AUTH_DELETE
        );
        ptln("   <div  id='$id' class='piPermissions'>");
		if(empty($acldata))
		{
			ptln("    <p>".$this->getLang('pi_no_permissions_found').'</p>');
			ptln("    </div>");
			return;
		}
        ptln("   <table>");
        $s = "<tr><th>".$this->getLang('pi_resource')."</th>";
        foreach($displayed_permissions as $p)
            $s .= "<th>".$this->getLang('acl_perm'.$p)."</td>";
        ptln($s."</tr>",6);

        
        $even = false;
        foreach($acldata as $item => $perm)
        {
            $additional_class = empty($this->explicitUserPermissions[$item]) ? "" : " explicitUserPermission";
            ptln('<tr class="'.($even?"even":"odd").'">', 6);
            if(preg_match('/\*\s*$/', $item))
                ptln('<td class="piItemNS'.$additional_class.'">'.$item.'</td>', 9);
            else
                ptln('<td class="piItemPage'.$additional_class.'">'.$item.'</td>',9);
            foreach($displayed_permissions as $p)
            {
                if($p & $perm)
                    ptln('<td class="piAllowed">X</td>', 9);
                else
                    ptln('<td class="piDenied">-</td>', 9);
            }
            $even = !$even;
        }
        ptln("   </table>");
        ptln("   </div>");
    }

 
    /**
     * This just gets a very rudimentary user and not very useful user list - 
     * only users who have special permissions in the ACL are listed. 
     * @return array This array is structured similar to the array returned by an auth class.
     */
    function _getUsersFromACL()
    {
        global $AUTH_ACL;
        $users = array();
        foreach($AUTH_ACL as $a)
        {
            // Don't parse comments
            if(preg_match('/^#/', $a))
                continue;
            if(preg_match('/^[^\s]+\s([^@\s]+)/', $a, $matches))
            {
                $usr_arr = array('name' => $matches[1], 'grps' => array());
                $users[$matches[1]] = $usr_arr;
            }
        }
        return $users;
    }

    /**
     * This function retrieves group names from the acl file.
     * Since none of the existing auth classes supports groups, I don't know 
     * what output to expect from them. I assume a two-dimensional hash similar 
     * to that from Auth->retrieveUsers.
     * @return array
     */
    function _getGroupsFromACL()
    {
        global $AUTH_ACL;
        $groups = array();
        foreach($AUTH_ACL as $a)
        {
            // Don't parse comments
            if(preg_match('/^#/', $a))
                continue;
            if(preg_match('/^[^\s]+\s@([^\s]+)/', $a, $matches))
            {
                $grp_arr = array('name' => $matches[1]);
                $groups[urldecode($matches[1])] = $grp_arr;
            }
        }
        return $groups;
    }

    /**
     * sets $this->aclGroupPermissions in the form of a[groupname][namespace/page_name]=permission
     */
    function _aclGroupPermissions()
    {
        global $AUTH_ACL;
        $gp = array();
        foreach($AUTH_ACL as $a)
        {
            // Don't parse comments
            if(preg_match('/^#/', $a))
                continue;
            if(preg_match('/^([^\s]+)\s@([^\s]+)\s(\d+)/', $a, $matches))
            {
                $gp[$matches[2]][$matches[1]] = $matches[3];
            }
        }
        $this->aclGroupPermissions = array();
        foreach($gp as $grpname => $permissions)
        {
            ksort($permissions);
            $this->aclGroupPermissions[$grpname] = $permissions;
        }
    }

    /**
     * sets $this->aclUserPermissions in the form of a[username][namespace/page_name]=permission
     */
    function _aclUserPermissions()
    {
        global $AUTH_ACL;
        $up = array();
        foreach($AUTH_ACL as $a)
        {
            // Don't parse comments
            if(preg_match('/^#/', $a))
                continue;
            if(preg_match('/^([^\s]+)\s([^@\s]+)\s(\d+)/', $a, $matches))
            {
                $up[$matches[2]][$matches[1]] = $matches[3];
            }
        }
        $this->aclUserPermissions = array();
        foreach($up as $usrname => $permissions)
        {
            ksort($permissions);
            $this->aclUserPermissions[$usrname] = $permissions;
        }
    }

    /**
     * Build an Array in $this->group2user that associates user names with users
     * The users are sorted by last name
     */
    function _group2user()
    {
        $g2u = array();
        foreach(array_keys($this->groups) as $g)
            $g2u[$g] = array();
        foreach($this->users as $username => $properties)
        {
            foreach($properties['grps'] as $grpname)
                $g2u[$grpname][$username] = array_pop(explode(' ', $properties['name'])); // Store last name of user here for Sorting
        }
        $this->group2user = array();
        foreach($g2u as $grpname => $users)
        {
            // Sort users in each group by last name
            asort($users);
            $this->group2user[$grpname] = array_keys($users);
        }
    }
    
    /**
     * Collects permission data for an individual user from teh ACL. It 
     * collects permission data from the groups of the user and from the 
     * explicitly assigned permissions for the user. 
     * The data is stored in the form of two arrays:
     * $this->userPermissions Namespace/Page => Permission pairs
     * $this->explicitUserPermissions Namespace/Page => Permission pairs
     */
    function _userPermissions($username)
    {
        // Build regular expression for the username an its groups
        $userdata = $this->auth->getUserData($username);
        $this->username = $userdata['name'];
        $search_string = preg_quote(auth_nameencode($username), '/');
        foreach($userdata['grps'] as $g)
            $search_string .= '|@'.preg_quote($g, '/');
        $perm_regex = '/^([^\s]+)\s('.$search_string.')\s(\d+)/';
        // Search through permissions
        global $AUTH_ACL;
        $up = array();
        // $for_user holds permissions that are assigned explicitly to the user
        $for_user = array(); 
        foreach($AUTH_ACL as $a)
        {
            // Don't parse comments
            if(preg_match('/^#/', $a))
                continue;
            if(preg_match($perm_regex, $a, $matches))
            {
                $up[$matches[1]] = (empty($up[$matches[1]])?0:$up[$matches[1]]) | $matches[3];
                if(substr($matches[2], 0, 1) != "@")
                    $for_user[$matches[1]] = $matches[3];
            }
        }
        ksort($up);
        ksort($for_user);
        $this->userPermissions = $up;
        $this->explicitUserPermissions = $for_user;
    }
}

